# SimMult_MX_Out

library(MASS)

remove(list=ls())

Hfun <-function(par,X,y,tune,reparam){
# Objective function for Gosh & Basu 2014 with tuning parameter tune 
# Negative Binomial regression case
n <- length(y); p = dim(X)[2]+1
if (!reparam) {Beta=par[1:p]; alpha = par[p+1]}
if ( reparam) {Beta=par[1:p]; alpha = par[p+1]^2}
size = 1/alpha
mux  = exp(cbind(1,X)%*%Beta)
Ymax = V = rep(0,n)
Ymax = qnbinom(1-1e-6,size=size,mu=mux)
for (i in 1:n) {
a = sum( dnbinom((0:Ymax[i]), size=size,mu=mux[i])^(1+tune))  
b = (1+1/tune)*dnbinom(y[i],  size=size,mu=mux[i])^tune
V[i] = a-b  }
mean(V)}

MDEr <- function(X,y,BetaI,alphaI,tune,reparam=FALSE) {
# Optimization of Hfun starting from BetaI, alphaI
# X without constant column
p = dim(X)[2]+1; n = length(y); bound ="no"
BetaA = rep(NA,p); alphaA=NA; val=NA; 
#
if (reparam==FALSE) {
 par  = c(BetaI,alphaI)
 ui   = diag(p+1)
 ci   = c(rep(-10,p), 0.01)
 MDE  = try(constrOptim(par,Hfun,NULL,ui,ci,X=X,y=y,tune=tune,reparam=FALSE))
 if (class(MDE)!="try-error") {
   BetaA  = MDE$par[1:p]; alphaA = MDE$par[p+1]; val=MDE$value}}
#
if (reparam==TRUE) {
 par  = c(BetaI,sqrt(alphaI))
 MDE  = try(optim(par,Hfun,NULL,X=X,y=y,tune=tune,reparam=TRUE))
 if (class(MDE)!="try-error") { 
  BetaA = MDE$par[1:p]; alphaA = MDE$par[p+1]^2; val=MDE$value}}
#
list(Beta=BetaA, alpha =alphaA, value=val)}

#-------------------------------------------------------------------------------
   
p = 6; Sigma0 = diag(1,p-1); Mu0 = c(0,0,0,0,0)
Beta0  = c(1.5, 0.5, 0.25,0,0,0); alpha0 = 0.8; size0  = 1/alpha0

epsilon = 0.15
xout = c(-2,-1,0,0,0)
yos  = c(0,1,2,10,20,30,40,50,60,70,100,120,180)
n     = 400
nsim  = 50

for (yo in yos) {
x     = matrix(rep(NA, (p-1)*n), ncol=p-1, byrow=FALSE)
y     = rep(NA, n)
nout  = round(epsilon*n)
m     = n-nout
yout  = yo

BetaX1 = BetaX2 = BetaX3 = BetaX4 = BetaX5 = BetaX6 = AlphaX = rep(NA,nsim)

# load M1
resdir <-  "C:/Users/Alfio/Desktop/SimulPrograms/Sim_Mult_Rev_Out/SimMult_M1_Out/"
resfile=paste(c(resdir, "SimMult_M1_Out_n", n, "_yout", yout, "_nsim", nsim, ".dmp"), collapse="") 
load(file=resfile )

for (i in 1:nsim) { 
set.seed(123+i)

x[1:m,]     = mvrnorm(m, Mu0, Sigma0)
if (nout!=0) x[(m+1):(m+nout),] =matrix(rep(xout,nout),ncol=p-1,byrow=TRUE)
Mu0xi = exp(cbind(1,x[,1:(p-1)])%*%Beta0)
y     = rnbinom(m, mu=Mu0xi, size=size0)
if (nout!=0) {y[(m+1):(m+nout)] = yout}

Beta.init  = c(Betam1[i],Betam2[i],Betam3[i],Betam4[i],Betam5[i],Betam6[i])
alpha.init = Alpham[i]

X = x; tune=0.4
if (!is.na(Betam1[i])) {
MDE <- MDEr(X,y,Beta.init,alpha.init,tune=tune,reparam=TRUE)
BetaX      = MDE$Beta[1:p]
alphaX     = MDE$alpha
bound      = MDE$bound; conv = MDE$conv
if (length(MDE)!=0) {
AlphaX[i]=alphaX
BetaX1[i]=BetaX[1]
BetaX2[i]=BetaX[2]
BetaX3[i]=BetaX[3]
BetaX4[i]=BetaX[4]
BetaX5[i]=BetaX[5]
BetaX6[i]=BetaX[6] 
}}

cat(yout, i, " ", BetaX1[i],BetaX2[i],BetaX3[i],BetaX4[i],BetaX5[i],BetaX6[i]," ", AlphaX[i], "\n")

} # end i 

resdir <-  "C:/Users/Alfio/Desktop/SimulPrograms/Sim_Mult_Rev_Out/SimMult_MX_Out/"
resfile=paste(c(resdir, "SimMult_MX_Out_n", n, "_yout", yout, "_nsim", nsim, ".dmp"), collapse="") 
save(BetaX1, BetaX2, BetaX3, BetaX4, BetaX5, BetaX6, AlphaX, file=resfile )

} # end yos

