# SimMult_wFIN_Out.s

library(MASS)
library(robustnegbin) 

#-------------------------------------------------------------------------------
   
p = 6; Sigma0 = diag(1,p-1); Mu0 = c(0,0,0,0,0)
Beta0  = c(1.5, 0.5, 0.25,0,0,0); alpha0 = 0.8; size0  = 1/alpha0

ctrl = glmrob.nb.control(
MCML = list(csil=0.05, csiu=0.95, iterate=2, tol=0.001, cutoff.adaptive=TRUE, gradient=FALSE, CoeffBound = 9), 
M    = list(mxT=300, tol=0.001, nint=10, verbose=FALSE, fortran=TRUE, ufact = 1.1), 
MRC  = list(nIterations=10, nAlternate=10, nGrid=25, tol=1e-06, standardize=FALSE) )

epsilon = 0.15
xout = c(-2,-1,0,0,0)
yos  = c(0,1,2,10,20,30,40,50,60,70,100,120,180)
n     = 400
nsim  = 50

for (yo in yos ) {
x     = matrix(rep(NA, (p-1)*n), ncol=p-1, byrow=FALSE)
y     = rep(NA, n)
nout  = round(epsilon*n)
m     = n-nout
yout  = yo

BetaI1 = BetaI2 = BetaI3 = BetaI4 = BetaI5 = BetaI6 = AlphaI = rep(0,nsim)
BetaC1 = BetaC2 = BetaC3 = BetaC4 = BetaC5 = BetaC6 = AlphaC = rep(0,nsim)

# load INI

resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Sim_Mult_Rev_Out/SimMult_INI_Out/"
resfile=paste(c(resdir, "SimMult_wINI_Out_n", n, "_yout", yout, "_nsim", nsim, ".dmp"), collapse="") 
load(resfile)

for (i in 1:nsim) { 

set.seed(123+i)
x[1:m,]     = mvrnorm(m, Mu0, Sigma0)
if (nout!=0) x[(m+1):(m+nout),] =matrix(rep(xout,nout),ncol=p-1,byrow=TRUE)
Mu0xi = exp(cbind(1,x[,1:(p-1)])%*%Beta0)
y     = rnbinom(m, mu=Mu0xi, size=size0)
if (nout!=0) {y[(m+1):(m+nout)] = yout}
XI = cbind(1,x[,1:(p-1)])
X  = XI[,2:p]
n  = length(y)

.RS=.Random.seed; u= runif(n); .Random.seed <<- .RS

Beta.init  = c(BetaI1[i],BetaI2[i],BetaI3[i],BetaI4[i],BetaI5[i],BetaI6[i])
alpha.init = AlphaI[i]
glm = glmrob.nb.fit(x=X,y=y,u=u, start=Beta.init, init.dispersion = alpha.init, final=TRUE, control=ctrl)
BetaC      = glm$coefficients
alphaC     = glm$dispersion
BetaC1[i]  = BetaC[1]
BetaC2[i]  = BetaC[2]
BetaC3[i]  = BetaC[3]
BetaC4[i]  = BetaC[4]
BetaC5[i]  = BetaC[5]
BetaC6[i]  = BetaC[6]
AlphaC[i]  = alphaC

cat(yout, i, " ", BetaC1[i],BetaC2[i],BetaC3[i],BetaC4[i],BetaC5[i],BetaC6[i]," ", AlphaC[i],"\n")
} # end i 

resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Sim_Mult_Rev_Out/SimMult_FIN_Out/"
resfile=paste(c(resdir, "SimMult_wFIN_Out_n", n, "_yout", yout, "_nsim", nsim, ".dmp"), collapse="") 
save(BetaC1, BetaC2, BetaC3, BetaC4, BetaC5, BetaC6, AlphaC, file=resfile )
} # end yo


