# Simul_MDE_noOut.s

library(MASS)

remove(list=ls())

Hfun <-function(par,X,y,tune,reparam){
# Objective function for Gosh & Basu 2014 with tuning parameter tune 
# Negative Binomial regression case
n <- length(y); p = dim(X)[2]+1
if (!reparam) {Beta=par[1:p]; alpha = par[p+1]}
if ( reparam) {Beta=par[1:p]; alpha = par[p+1]^2}
size = 1/alpha
mux  = exp(cbind(1,X)%*%Beta)
Ymax = V = rep(0,n)
Ymax = qnbinom(1-1e-6,size=size,mu=mux)
for (i in 1:n) {
a = sum( dnbinom((0:Ymax[i]), size=size,mu=mux[i])^(1+tune))  
b = (1+1/tune)*dnbinom(y[i],  size=size,mu=mux[i])^tune
V[i] = a-b  }
mean(V)}

MDEr <- function(X,y,BetaI,alphaI,tune,reparam=FALSE) {
# Optimization of Hfun starting from BetaI, alphaI
# X without constant column
p = dim(X)[2]+1; n = length(y); bound ="no"
BetaA = rep(NA,p); alphaA=NA; val=NA; 
#
if (reparam==FALSE) {
 par  = c(BetaI,alphaI)
 ui   = diag(p+1)
 ci   = c(rep(-10,p), 0.01)
 MDE  = try(constrOptim(par,Hfun,NULL,ui,ci,X=X,y=y,tune=tune,reparam=FALSE))
 if (class(MDE)!="try-error") {
   BetaA  = MDE$par[1:p]; alphaA = MDE$par[p+1]; val=MDE$value}}
#
if (reparam==TRUE) {
 par  = c(BetaI,sqrt(alphaI))
 MDE  = try(optim(par,Hfun,NULL,X=X,y=y,tune=tune,reparam=TRUE))
 if (class(MDE)!="try-error") { 
  BetaA = MDE$par[1:p]; alphaA = MDE$par[p+1]^2; val=MDE$value}}
#
list(Beta=BetaA, alpha =alphaA, value=val)}

#-------------------------------------------------------------------------------
   
p = 6; Sigma0 = diag(1,p-1); Mu0 = c(0,0,0,0,0)
Beta0  = c(1.5, 0.5, 0.25,0,0,0); alpha0 = 0.8; size0  = 1/alpha0

nsim = 1000

for (n in c(100) ) {
x     = matrix(rep(NA, (p-1)*n), ncol=p-1, byrow=FALSE)
y     = rep(NA, n)

BetaD1 = BetaD2 = BetaD3 = BetaD4 = BetaD5 = BetaD6 = AlphaD = rep(0,nsim)

# load ML
resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Simul_Mult_Rev_NoOut/SimMult_ML_noOut/"
resfile=paste(c(resdir, "SimMult_ML8_noOut_n", n, ".dmp"), collapse="") 
load(file=resfile )

#---

for (i in 1:nsim) { 
set.seed(123+i)
x[1:n,]     = mvrnorm(n, Mu0, Sigma0)
Mu0xi = exp(cbind(1,x[,1:(p-1)])%*%Beta0)
y     = rnbinom(n, mu=Mu0xi, size=size0)

Beta.init  = c(BetaG1[i],BetaG2[i],BetaG3[i],BetaG4[i],BetaG5[i],BetaG6[i])
alpha.init = AlphaG[i]

X = x; tune=0.4
MDE <- MDEr(X,y,Beta.init,alpha.init,tune=tune,reparam=TRUE)
BetaD      = MDE$Beta[1:p]
alphaD     = MDE$alpha
bound      = MDE$bound; conv = MDE$conv
if (length(MDE)!=0) {
AlphaD[i]=alphaD
BetaD1[i]=BetaD[1]
BetaD2[i]=BetaD[2]
BetaD3[i]=BetaD[3]
BetaD4[i]=BetaD[4]
BetaD5[i]=BetaD[5]
BetaD6[i]=BetaD[6] 
}

cat(n, i, " ", BetaD1[i],BetaD2[i],BetaD3[i],BetaD4[i],BetaD5[i],BetaD6[i]," ", AlphaD[i], "\n")

} # end i 

resdir  = "C:/Users/Alfio/Desktop/SimulPrograms/Simul_Mult_Rev_NoOut/SimMult_MDE_noOut/"
resfile = paste(c(resdir, "SimMult_MDE8_noOut_n", n,"_",tune, ".dmp"), collapse="") 
save(BetaD1, BetaD2, BetaD3, BetaD4, BetaD5, BetaD6, AlphaD, file=resfile )

} # end n


