# SimMult_wFIN_noOut.s

library(MASS)
library(robustnegbin) 

#-------------------------------------------------------------------------------
   
p = 6; Sigma0 = diag(1,p-1); Mu0 = c(0,0,0,0,0)
Beta0  = c(1.5, 0.5, 0.25,0,0,0); alpha0 = 0.8; size0  = 1/alpha0

ctrl = glmrob.nb.control(
MCML = list(csil=0.05, csiu=0.95, iterate=2, tol=0.001, cutoff.adaptive=TRUE, gradient=FALSE, CoeffBound = 9), 
M    = list(mxT=300, tol=0.001, nint=10, verbose=FALSE, fortran=TRUE, ufact = 1.1), 
MRC  = list(nIterations=10, nAlternate=10, nGrid=25, tol=1e-06, standardize=FALSE) )

nn = c(100,400,1000,2000)
nsim = 1000

for (n in nn) {
BetaC1 = BetaC2 = BetaC3 = BetaC4 = BetaC5 = BetaC6 = AlphaC = rep(NA,nsim)
Tl = Tu = Nit = rep(NA,nsim)
x     = matrix(rep(NA, (p-1)*n), ncol=p-1, byrow=FALSE)
y     = rep(NA, n)

for (i in 1:nsim) { 
set.seed(123+i)
x[1:n,]     = mvrnorm(n, Mu0, Sigma0)
Mu0xi = exp(cbind(1,x[,1:(p-1)])%*%Beta0)
y     = rnbinom(n, mu=Mu0xi, size=size0)

XI = cbind(1,x[,1:(p-1)])
X  = XI[,2:p]
n  = length(y)

.RS=.Random.seed; u= runif(n); .Random.seed <<- .RS

resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Simul_Mult_Rev_NoOut/SimMult_INI_noOut/"
resfile=paste(c(resdir, "SimMult_wINI8_noOut_n", n, ".dmp"), collapse="") 
load(resfile)

Beta.init  = c(BetaI1[i],BetaI2[i],BetaI3[i],BetaI4[i],BetaI5[i],BetaI6[i])
alpha.init = AlphaI[i]

if (!is.na(BetaI1[i]) ) {
glm = glmrob.nb.fit(X,y,u=u, start=Beta.init, init.dispersion = alpha.init, final=TRUE, control=ctrl)
BetaC      = glm$coefficients
alphaC     = glm$dispersion
BetaC1[i]  = BetaC[1]
BetaC2[i]  = BetaC[2]
BetaC3[i]  = BetaC[3]
BetaC4[i]  = BetaC[4]
BetaC5[i]  = BetaC[5]
BetaC6[i]  = BetaC[6]
AlphaC[i]  = alphaC
Tl[i]      = glm$obj.final$a
Tu[i]      = glm$obj.final$b
Nit[i]     = glm$obj.final$nit
}

cat(n, i, " ", BetaC1[i],BetaC2[i],BetaC3[i],BetaC4[i],BetaC5[i],BetaC6[i]," ", AlphaC[i], " ",Tl[i],Tu[i],Nit[i],"\n")	
} # end i

resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Simul_Mult_Rev_NoOut/SimMult_FIN_noOut/"
resfile=paste(c(resdir, "SimMult_wFIN8_noOut_n", n, ".dmp"), collapse="") 
save(BetaC1, BetaC2, BetaC3, BetaC4, BetaC5, BetaC6, AlphaC, Tl, Tu,file=resfile )

} # end n

