# SimMult_wINI_CML_noOut.s

library(MASS)
library(robustnegbin) 

#-------------------------------------------------------------------------------
   
p = 6; Sigma0 = diag(1,p-1); Mu0 = c(0,0,0,0,0)
Beta0  = c(1.5, 0.5, 0.25,0,0,0); alpha0 = 0.8; size0  = 1/alpha0

ctrl = glmrob.nb.control(
MCML = list(csil=0.05, csiu=0.95, iterate=2, tol=0.001, cutoff.adaptive=TRUE, gradient=FALSE, CoeffBound = 9), 
M    = list(mxT=300, tol=0.001, nint=10, verbose=FALSE, fortran=TRUE, ufact = 1.1), 
MRC  = list(nIterations=10, nAlternate=10, nGrid=25, tol=1e-06, standardize=FALSE) )

nn = c(100,400,1000,2000)
nsim = 1000

for (n in nn) {
BetaI1 = BetaI2 = BetaI3 = BetaI4 = BetaI5 = BetaI6 = AlphaI = rep(NA,nsim)
x     = matrix(rep(NA, (p-1)*n), ncol=p-1, byrow=FALSE)
y     = rep(NA, n)

for (i in 1:nsim) { 
set.seed(123+i)
x[1:n,] = mvrnorm(n, Mu0, Sigma0)
Mu0xi   = exp(cbind(1,x[,1:(p-1)])%*%Beta0)
y       = rnbinom(n, mu=Mu0xi, size=size0)
XI = cbind(1,x[,1:(p-1)])
X  = XI[,2:p]
n  = length(y)

.RS=.Random.seed; u= runif(n); .Random.seed <<- .RS

# Compute the x-weights

#if (i !=330 ) {
Xrc <- cov.rob(X,quantile.used=floor(0.8*dim(X)[1]),nsamp=50000) # MVE keeping 80% observations 
D2  <- mahalanobis(X, center = Xrc$center, cov = Xrc$cov)
qchi2 <- qchisq(p = 0.95, df = dim(X)[2])
wx <- ifelse(D2 <= qchi2, 1, 0)
ind.outx = (1:n)[wx==0]

# Remove x-outliers and compute wINI

Xr = X[-ind.outx,]
yr = y[-ind.outx]
ur = u[-ind.outx]
glm = glmrob.nb.fit(x=Xr,y=yr,u=ur, start=NULL, init.dispersion = NULL, final=FALSE, control=ctrl)
BetaI      = glm$coefficients
alphaI     = glm$dispersion
BetaI1[i]  = BetaI[1]
BetaI2[i]  = BetaI[2]
BetaI3[i]  = BetaI[3]
BetaI4[i]  = BetaI[4]
BetaI5[i]  = BetaI[5]
BetaI6[i]  = BetaI[6]
AlphaI[i]  = alphaI
#}

cat(n, i, " ", BetaI1[i],BetaI2[i],BetaI3[i],BetaI4[i],BetaI5[i],BetaI6[i],"   ", AlphaI[i], "\n")

} # end i

resdir <- "C:/Users/Alfio/Desktop/SimulPrograms/Simul_Mult_Rev_NoOut/SimMult_INI_noOut/"
resfile=paste(c(resdir, "SimMult_wINIx_noOut_n", n, ".dmp"), collapse="") 
save(BetaI1,BetaI2,BetaI3,BetaI4,BetaI5,BetaI6,AlphaI, file=resfile )

} # end n

